@@ -38,6 +38,12 @@ class Gencontrol(Base):
             'check-size': config.SchemaItemInteger(),
             'check-size-with-dtb': config.SchemaItemBoolean(),
             'check-uncompressed-size': config.SchemaItemInteger(),
+            'depends': config.SchemaItemList(','),
+            'provides': config.SchemaItemList(','),
+            'suggests': config.SchemaItemList(','),
+            'recommends': config.SchemaItemList(','),
+            'conflicts': config.SchemaItemList(','),
+            'breaks': config.SchemaItemList(','),
         },
         'relations': {
         },
@@ -52,6 +58,12 @@ class Gencontrol(Base):
         }
     }
 
+    env_flags = [
+        ('DEBIAN_KERNEL_DISABLE_DEBUG', 'disable_debug', 'debug infos'),
+        ('DEBIAN_KERNEL_DISABLE_INSTALLER', 'disable_installer', 'installer modules'),
+        ('DEBIAN_KERNEL_DISABLE_SIGNED', 'disable_signed', 'signed code'),
+    ]
+
     def __init__(self, config_dirs=["debian/config"],
                  template_dirs=["debian/templates"]):
         super(Gencontrol, self).__init__(
@@ -61,6 +73,17 @@ class Gencontrol(Base):
         self.process_changelog()
         self.config_dirs = config_dirs
 
+        for env, attr, desc in self.env_flags:
+            setattr(self, attr, False)
+            if os.getenv(env):
+                if self.changelog[0].distribution == 'UNRELEASED':
+                    import warnings
+                    warnings.warn(f'Disable {desc} on request ({env} set)')
+                    setattr(self, attr, True)
+                else:
+                    raise RuntimeError(
+                        'Unable to disable {desc} in release build ({env} set)')
+
     def _setup_makeflags(self, names, makeflags, data):
         for src, dst, optional in names:
             if src in data or not optional:
@@ -87,16 +110,7 @@ class Gencontrol(Base):
 
         self.installer_packages = {}
 
-        if os.getenv('DEBIAN_KERNEL_DISABLE_INSTALLER'):
-            if self.changelog[0].distribution == 'UNRELEASED':
-                import warnings
-                warnings.warn('Disable installer modules on request '
-                              '(DEBIAN_KERNEL_DISABLE_INSTALLER set)')
-            else:
-                raise RuntimeError(
-                    'Unable to disable installer modules in release build '
-                    '(DEBIAN_KERNEL_DISABLE_INSTALLER set)')
-        elif self.config.merge('packages').get('installer', True):
+        if not self.disable_installer and self.config.merge('packages').get('installer', True):
             # Add udebs using kernel-wedge
             kw_env = os.environ.copy()
             kw_env['KW_DEFCONFIG_DIR'] = 'debian/installer'
@@ -126,8 +140,11 @@ class Gencontrol(Base):
             # configuration errors before building linux-signed.
             build_signed = {}
             for arch in arches:
-                build_signed[arch] = self.config.merge('build', arch) \
-                                                .get('signed-code', False)
+                if not self.disable_signed:
+                    build_signed[arch] = self.config.merge('build', arch) \
+                                                    .get('signed-code', False)
+                else:
+                    build_signed[arch] = False
 
             for package in udeb_packages:
                 # kernel-wedge currently chokes on Build-Profiles so add it now
@@ -240,8 +257,11 @@ class Gencontrol(Base):
             makeflags['ABINAME'] = vars['abiname'] = \
                 self.abiname_version + abiname_part
 
-        build_signed = self.config.merge('build', arch) \
-                                  .get('signed-code', False)
+        if not self.disable_signed:
+            build_signed = self.config.merge('build', arch) \
+                                      .get('signed-code', False)
+        else:
+            build_signed = False
 
         # Some userland architectures require kernels from another
         # (Debian) architecture, e.g. x32/amd64.
@@ -371,11 +391,13 @@ class Gencontrol(Base):
                                                flavour)
         config_entry_description = self.config.merge('description', arch,
                                                      featureset, flavour)
-        config_entry_image = self.config.merge('image', arch, featureset,
-                                               flavour)
         config_entry_relations = self.config.merge('relations', arch,
                                                    featureset, flavour)
 
+        def config_entry_image(key, *args, **kwargs):
+            return self.config.get_merge(
+                'image', arch, featureset, flavour, key, *args, **kwargs)
+
         compiler = config_entry_base.get('compiler', 'gcc')
 
         # Work out dependency from linux-headers to compiler.  Drop
@@ -403,10 +425,11 @@ class Gencontrol(Base):
         image_fields = {'Description': PackageDescription()}
         for field in ('Depends', 'Provides', 'Suggests', 'Recommends',
                       'Conflicts', 'Breaks'):
-            image_fields[field] = PackageRelation(config_entry_image.get(
-                field.lower(), None), override_arches=(arch,))
+            image_fields[field] = PackageRelation(
+                config_entry_image(field.lower(), None),
+                override_arches=(arch,))
 
-        generators = config_entry_image['initramfs-generators']
+        generators = config_entry_image('initramfs-generators')
         group = PackageRelationGroup()
         for i in generators:
             i = config_entry_relations.get(i, i)
@@ -419,7 +442,7 @@ class Gencontrol(Base):
             item.arches = [arch]
         image_fields['Depends'].append(group)
 
-        bootloaders = config_entry_image.get('bootloaders')
+        bootloaders = config_entry_image('bootloaders', None)
         if bootloaders:
             group = PackageRelationGroup()
             for i in bootloaders:
@@ -448,7 +471,10 @@ class Gencontrol(Base):
 
         packages_own = []
 
-        build_signed = config_entry_build.get('signed-code')
+        if not self.disable_signed:
+            build_signed = config_entry_build.get('signed-code')
+        else:
+            build_signed = False
 
         image = self.templates[build_signed and "control.image-unsigned"
                                or "control.image"]
@@ -470,18 +496,10 @@ class Gencontrol(Base):
         if config_entry_build.get('vdso', False):
             makeflags['VDSO'] = True
 
-        build_debug = config_entry_build.get('debug-info')
-
-        if os.getenv('DEBIAN_KERNEL_DISABLE_DEBUG'):
-            if self.changelog[0].distribution == 'UNRELEASED':
-                import warnings
-                warnings.warn('Disable debug infos on request '
-                              '(DEBIAN_KERNEL_DISABLE_DEBUG set)')
-                build_debug = False
-            else:
-                raise RuntimeError(
-                    'Unable to disable debug infos in release build '
-                    '(DEBIAN_KERNEL_DISABLE_DEBUG set)')
+        if not self.disable_debug:
+            build_debug = config_entry_build.get('debug-info')
+        else:
+            build_debug = False
 
         if build_debug:
             makeflags['DEBUG'] = True
